/**
 * @license Copyright (c) 2017-2025, CKSource Holding sp. z o.o. All rights reserved.
 * For licensing, see LICENSE.md.
 */

'use strict';

const fs = require( 'fs' );

/**
 * Copies a specified file from the `source` path to the `destination` path.
 *
 * @param {String} source
 * @param {String} destination
 * @return {Promise}
 */
module.exports = function copyFile( source, destination ) {
	return new Promise( ( resolve, reject ) => {
		const readStream = fs.createReadStream( source );

		// In order to catch an error if `source` points to non-existing file, we need to wait for
		// the `#open` event emitted by the read readStream.
		readStream.on( 'open', () => {
			const writeStream = fs.createWriteStream( destination );

			readStream.pipe( writeStream );

			writeStream.on( 'error', reject );
			writeStream.on( 'finish', resolve );
		} );

		readStream.on( 'error', reject );
	} );
};
