/**
 * @license Copyright (c) 2017-2025, CKSource Holding sp. z o.o. All rights reserved.
 * For licensing, see LICENSE.md.
 */

'use strict';

const upath = require( 'upath' );
const mkdirp = require( 'mkdirp' );
const copyFile = require( '../helpers/copy-file' );

/**
 * @returns {Promise}
 */
module.exports = function copyProjectIcons( projectGlobals, buildDirectory ) {
	const availableProjects = Object.keys( projectGlobals ).filter( projectName => projectName !== 'common' );

	let promise = Promise.resolve();

	for ( const project of availableProjects ) {
		const projectDetails = projectGlobals[ project ];
		const destinationIconsPath = upath.join( buildDirectory, projectDetails.BASE_PATH, 'assets', 'icons' );

		promise = promise.then( () => mkdirp( destinationIconsPath ) );

		for ( const [ sourceFile, destinationName ] of projectDetails._icons ) {
			const fullSourcePath = upath.join( projectDetails.config.projectRootPath, 'node_modules', sourceFile );
			const fullDestinationPath = upath.join( destinationIconsPath, destinationName );

			promise = promise.then( () => {
				return copyFile( fullSourcePath, fullDestinationPath )
					.catch( () => {
						console.warn(
							`⚠️ The "${ sourceFile }" icon cannot be copied because it does not exist in "${ fullSourcePath }".`
						);
					} );
			} );
		}
	}

	return promise;
};
