/**
 * @license Copyright (c) 2017-2025, CKSource Holding sp. z o.o. All rights reserved.
 * For licensing, see LICENSE.md.
 */

'use strict';

const getProjectConfig = require( './get-project-config' );
const upath = require( 'upath' );
const fs = require( 'fs' );
const chalk = require( 'chalk' );

module.exports = async ( {
	mainConfig,
	rootPath,
	isSingleProject = false
} ) => {
	const configs = [];

	if ( isSingleProject ) {
		configs.push( await getProjectConfig( rootPath, { skipLiveSnippets: true } ) );
	} else {
		const promises = mainConfig.projects.map( projectPath => {
			return getProjectConfig( upath.join( rootPath, projectPath ), { skipLiveSnippets: true } );
		} );
		configs.push( ...await Promise.all( promises ) );
	}

	configs
		.filter( config => config.version !== 'latest' )
		.forEach( config => {
			createSymbolicLink( config );
		} );

	return Promise.resolve();

	function createSymbolicLink( config ) {
		const destinationPath = upath.join( rootPath, 'build', 'docs', config.slug, 'latest' );

		if ( !fs.existsSync( destinationPath ) ) {
			// The 3rd argument to the fs.symlinkSync() method is the link type and it is only available
			// on Windows (it is ignored on other platforms). Creating a symlink to a directory with link
			// type = 'dir' requires admin privileges on Windows. The only way to create a directory symlink
			// without admin privileges on Windows is to use the 'junction' link type.
			fs.symlinkSync( config.version, destinationPath, 'junction' );
			console.log( `Symlink for ${ chalk.cyanBright( config.name ) } ${ chalk.magentaBright( './' + config.version ) } - created. ` );
		}
	}
};
