/**
 * @license Copyright (c) 2017-2025, CKSource Holding sp. z o.o. All rights reserved.
 * For licensing, see LICENSE.md.
 */

'use strict';

const fs = require( 'fs' );
const mkdirp = require( 'mkdirp' );
const upath = require( 'upath' );

/**
 * Saves .html files to file system.
 *
 * @param {Array.<HtmlFile>} files HtmlFile objects containing content to be written to .html files to file system.
 * @param {String} destinationDir Additional destination path. Additional because files already have its own path.
 * @returns {Promise}
 */
module.exports = ( files, destinationDir = '.' ) => {
	const promises = [];

	for ( const file of files ) {
		const fileDir = upath.join( destinationDir, file.dirname );

		const promise = mkdirp( upath.resolve( fileDir ) )
			.then( () => {
				return new Promise( ( resolve, reject ) => {
					fs.writeFile( upath.join( fileDir, file.basename ), file.content, err => {
						if ( err ) {
							return reject( err );
						}

						return resolve();
					} );
				} );
			} );

		promises.push( promise );
	}

	return Promise.all( promises );
};
